/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│vi: set net ft=c ts=2 sts=2 sw=2 fenc=utf-8                                :vi│
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2020 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/bits/safemacros.internal.h"
#include "libc/sock/sock.h"
#include "libc/sysv/consts/af.h"
#include "libc/sysv/consts/inaddr.h"
#include "libc/testlib/testlib.h"

TEST(inet_pton, testLocalhost) {
  uint8_t addr[4] = {255, 255, 255, 255};
  EXPECT_EQ(1, inet_pton(AF_INET, "127.0.0.1", &addr));
  EXPECT_EQ(127, addr[0]);
  EXPECT_EQ(0, addr[1]);
  EXPECT_EQ(0, addr[2]);
  EXPECT_EQ(1, addr[3]);
}

TEST(inet_pton, testAny) {
  uint8_t addr[4] = {255, 255, 255, 255};
  EXPECT_EQ(1, inet_pton(AF_INET, "0.0.0.0", &addr));
  EXPECT_EQ(0, addr[0]);
  EXPECT_EQ(0, addr[1]);
  EXPECT_EQ(0, addr[2]);
  EXPECT_EQ(0, addr[3]);
}

TEST(inet_pton, testShortAddress_doesntFillFullValue) {
  uint8_t addr[4] = {255, 255, 255, 255};
  EXPECT_EQ(0, inet_pton(AF_INET, "127.0.0", &addr));
  EXPECT_EQ(127, addr[0]);
  EXPECT_EQ(0, addr[1]);
  EXPECT_EQ(0, addr[2]);
  EXPECT_EQ(255, addr[3]);
}

TEST(inet_pton, testOverflow_stopsParsing) {
  uint8_t addr[4] = {255, 255, 255, 255};
  EXPECT_EQ(0, inet_pton(AF_INET, "0.300.0", &addr));
  EXPECT_EQ(0, addr[0]);
  EXPECT_EQ(255, addr[1]);
  EXPECT_EQ(255, addr[2]);
  EXPECT_EQ(255, addr[3]);
}

TEST(inet_pton, testBadChar_stopsParsing) {
  uint8_t addr[4] = {255, 255, 255, 255};
  EXPECT_EQ(0, inet_pton(AF_INET, "127-.0.0", &addr));
  EXPECT_EQ(127, addr[0]);
  EXPECT_EQ(255, addr[1]);
  EXPECT_EQ(255, addr[2]);
  EXPECT_EQ(255, addr[3]);
}

TEST(inet_pton, testBadFamily_returnsNegAndChangesNothing) {
  uint8_t addr[4] = {255, 255, 255, 255};
  EXPECT_EQ(-1, inet_pton(666, "127.0.0.1", &addr));
  EXPECT_EQ(255, addr[0]);
  EXPECT_EQ(255, addr[1]);
  EXPECT_EQ(255, addr[2]);
  EXPECT_EQ(255, addr[3]);
}
